import os
import time
import datetime
import bisect
from collections import OrderedDict

from reportWaysonQi.api import make_report
from reportWaysonQi import HTMLTestRunner


def pytest_addoption(parser):
    group = parser.getgroup('terminal reporting')
    group.addoption('--pytest_report', action='store', dest='pytest_report', metavar='path', default=None,
                    help='create html report file at given path.')
    group.addoption('--pytest_title', action='store', dest='pytest_title', metavar='path', default="GLR Automation Test Report",
                    help='given title for report.')
    group.addoption('--pytest_desc', action='store', dest='pytest_desc', metavar='path', default="This is the GUI automation test for the GLR project-Wayson Qi",
                    help='given desc for report..')
    group.addoption('--pytest_theme', action='store', dest='pytest_theme', metavar='path', default=None,
                    help='given theme for report.')
    group.addoption('--pytest_stylesheet', action='store', dest='pytest_stylesheet', metavar='path', default=None,
                    help='given css file path for report.')
    group.addoption('--pytest_htmltemplate', action='store', dest='pytest_htmltemplate', metavar='path', default=None,
                    help='given html file path for report.')
    group.addoption('--pytest_javascript', action='store', dest='pytest_javascript', metavar='path', default=None,
                    help='given js file path for report.')


def pytest_configure(config):
    htmlpath = config.getoption('pytest_report')
    if htmlpath:
        # for csspath in config.getoption('css'):
        #     open(csspath)
        if not hasattr(config, 'slaveinput'):
            # prevent opening htmlpath on slave nodes (xdist)
            config._html = HTMLReport(htmlpath, config)
            config.pluginmanager.register(config._html)


def pytest_unconfigure(config):
    html = getattr(config, '_html', None)
    if html:
        del config._html
        config.pluginmanager.unregister(html)


class TestResult:
    def __init__(self, outcome, report):
        self.outcome = outcome
        self.report = report

        # print(outcome)
        # print(report._to_json())

    def output(self, cid, tid):
        if self.outcome.startswith('X'):
            status = 'fail'
            status_code = 1
        elif self.outcome == "Passed":
            status = 'pass'
            status_code = 0
        elif self.outcome == "Failed":
            status = 'fail'
            status_code = 1
        elif self.outcome == "Error":
            status = 'error'
            status_code = 2
        elif self.outcome == "Skipped":
            status = 'skip'
            status_code = 3
        else:
            status = 'pass'
            status_code = 0

        output = "%s\r\n%s" % (self.outcome, self.report.longrepr) if status != 'pass' else ""

        return {
            "has_output": output and True or False,
            "tid": "test%s.%s.%s" % (status, cid, tid),
            "desc": self.report.nodeid.split("::")[-1],
            "output": output,
            "status": status,
            "status_code": status_code
        }


class HTMLReport(object):

    def __init__(self, html_file, config):
        html_file = os.path.expanduser(os.path.expandvars(html_file))
        self.html_file = os.path.abspath(html_file)
        self.results = []
        self.errors = self.failed = 0
        self.passed = self.skipped = 0
        self.xfailed = self.xpassed = 0
        has_rerun = config.pluginmanager.hasplugin('rerunfailures')
        self.rerun = 0 if has_rerun else None
        self.config = config

    def _appendrow(self, outcome, report):
        result = TestResult(outcome, report)
        self.results.append(result)

    def append_passed(self, report):
        if report.when == 'call':
            if hasattr(report, "wasxfail"):
                # pytest < 3.0 marked xpasses as failures
                self.xpassed += 1
                self._appendrow('XPassed', report)
            else:
                self.passed += 1
                self._appendrow('Passed', report)

    def append_failed(self, report):
        if getattr(report, 'when', None) == "call":
            if hasattr(report, "wasxfail"):
                self.xfailed += 1
                self._appendrow('XFailed', report)
            else:
                message = report.longrepr.reprcrash.message
                if message.startswith('assert'):    # assert Error
                    self.failed += 1
                    self._appendrow('Failed', report)
                else:
                    self.errors += 1
                    self._appendrow('Error', report)
        else:
            self.errors += 1
            self._appendrow('Error', report)

    def append_skipped(self, report):
        if hasattr(report, "wasxfail"):
            self.xfailed += 1
            self._appendrow('XFailed', report)
        else:
            self.skipped += 1
            self._appendrow('Skipped', report)

    def append_other(self, report):
        # For now, the only "other" the plugin give support is rerun
        self.rerun += 1
        self._appendrow('Rerun', report)

    def _generate_detail(self):
        tests = []
        sorted_result = self.sort_result()
        for cid, (cls, cls_results) in enumerate(sorted_result, 1):
            # subtotal for a class
            np = nf = ne = ns = 0
            for result in cls_results:
                if result.outcome == "Passed":  # pass
                    np += 1
                elif result.outcome in ["Failed", "XPassed", "XFailed"]:    # fail
                    nf += 1
                elif result.outcome == "Error":    # error
                    ne += 1
                elif result.outcome == "Skipped":       # skip
                    ns += 1

            # format class description
            name = cls
            doc = ""
            desc = '%s: %s' % (name, doc) if doc else name

            test = {
                'summary': {
                    'desc': desc,
                    'count': np + nf + ne + ns,
                    'pass': np,
                    'fail': nf,
                    'error': ne,
                    'skip': ns,
                    'cid': 'testclass%s' % cid,
                    'status': (ne and "error") or (nf and "fail") or (ns and "skip") or "pass"
                }, 'detail': []
            }

            for tid, result in enumerate(cls_results, 1):
                test['detail'].append(result.output(cid, tid))

            tests.append(test)

        return tests

    def sort_result(self):
        rmap = {}
        for result in self.results:
            cls = result.report.nodeid.split("::")[1]
            # cls = cls_path.split("/")[-1]
            rmap.setdefault(cls, []).append(result)
        return rmap.items()

    def _generate_report(self, session):
        suite_stop_time = datetime.datetime.now()
        duration = (suite_stop_time - self.suite_start_time).total_seconds()
        count = self.passed + self.failed + self.xpassed + self.xfailed + self.skipped + self.errors
        environment = self._generate_environment(session.config)
        tests = self._generate_detail()

        report_content = {
            "generator": "PyTestReport %s" % HTMLTestRunner.__version__,
            "title": "%s" % self.config.getoption('pytest_title'),
            "description": "%s" % self.config.getoption('pytest_desc'),
            "environment": environment,
            "report_summary": {
                "start_time": self.suite_start_time.strftime('%Y-%m-%d %H:%M:%S'),
                "duration": duration,
                "suite_count": len(tests),
                "status": {
                    "pass": self.passed,
                    "fail": self.failed + self.xfailed + self.xpassed,
                    "error": self.errors,
                    "skip": self.skipped,
                    "count": count
                }
            }, "report_detail": {
                "tests": tests,
                "count": count,
                "pass": self.passed,
                "fail": self.failed + self.xfailed + self.xpassed,
                "error": self.errors,
                "skip": self.skipped,
            }
        }

        return report_content

    def _generate_environment(self, config):
        if not hasattr(config, '_metadata') or config._metadata is None:
            return []

        return config._metadata

    def _save_report(self, report_content, theme=None, stylesheet=None, htmltemplate=None, javascript=None):
        dir_name = os.path.dirname(self.html_file)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)

        with open(self.html_file, 'wb') as fp:
            # make_report(fp, report_content, theme=theme, stylesheet=stylesheet, htmltemplate=htmltemplate,
            #             javascript=javascript)

            test_runner = HTMLTestRunner.HTMLTestRunner(fp, theme=theme, stylesheet=stylesheet,
                                         htmltemplate=htmltemplate, javascript=javascript)
            report_content['stylesheet'] = test_runner.get_stylesheet()
            report_content['javascript'] = test_runner.get_javascript()
            return test_runner.generate_report(report_content)

    def pytest_runtest_logreport(self, report):
        if report.passed:
            self.append_passed(report)
        elif report.failed:
            self.append_failed(report)
        elif report.skipped:
            self.append_skipped(report)
        else:
            self.append_other(report)

    def pytest_collectreport(self, report):
        if report.failed:
            self.append_failed(report)

    def pytest_sessionstart(self, session):
        self.suite_start_time = datetime.datetime.now()

    def pytest_sessionfinish(self, session):
        report_data = self._generate_report(session)
        theme = self.config.getoption('pytest_theme')
        stylesheet = self.config.getoption('pytest_stylesheet')
        htmltemplate = self.config.getoption('pytest_htmltemplate')
        javascript = self.config.getoption('pytest_javascript')
        self._save_report(report_data, theme, stylesheet, htmltemplate, javascript)

    def pytest_terminal_summary(self, terminalreporter):
        terminalreporter.write_sep('-', 'generated html file: {0}'.format(
            self.html_file))




__original_author__ = ""
__update_author__ = ""
__version__ = ""

import datetime
import io
import os
import sys
import unittest
from xml.sax import saxutils
from jinja2 import Environment, PackageLoader

env = Environment(loader=PackageLoader('reportWaysonQi', 'templates'))


class OutputRedirector(object):

    def __init__(self, fp):
        self.fp = fp

    def write(self, s):
        self.fp.write(bytes(s, 'UTF-8'))

    def writelines(self, lines):
        self.fp.writelines(lines)

    def flush(self):
        self.fp.flush()


stdout_redirector = OutputRedirector(sys.stdout)
stderr_redirector = OutputRedirector(sys.stderr)


class TemplateMixin(object):


    DIR = os.path.dirname(os.path.abspath(__file__))
    STYLESHEET_DIR = os.path.join(DIR, 'static', 'css')
    JAVASCRIPT_DIR = os.path.join(DIR, 'static', 'js')
    HTMLTEMPLATE_DIR = os.path.join(DIR, 'templates')

    STATUS = {
        0: 'pass',
        1: 'fail',
        2: 'error',
        3: 'skip'
    }

    DEFAULT_TITLE = 'GLR Automation Test Report'
    DEFAULT_DESCRIPTION = ''


TestResult = unittest.TestResult


class _TestResult(TestResult):

    def __init__(self, verbosity=1):
        super().__init__()
        self.stdout0 = None
        self.stderr0 = None
        self.outputBuffer = None

        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0
        self.skip_count = 0

        self.verbosity = verbosity

        self.result = []

    def startTest(self, test):
        TestResult.startTest(self, test)

        # just one buffer for both stdout and stderr
        self.outputBuffer = io.BytesIO()
        stdout_redirector.fp = self.outputBuffer
        stderr_redirector.fp = self.outputBuffer
        self.stdout0 = sys.stdout
        self.stderr0 = sys.stderr
        sys.stdout = stdout_redirector
        sys.stderr = stderr_redirector

    def complete_output(self):

        if self.stdout0:
            sys.stdout = self.stdout0
            sys.stderr = self.stderr0
            self.stdout0 = None
            self.stderr0 = None
        return self.outputBuffer.getvalue()

    def stopTest(self, test):
        self.complete_output()

    def addSuccess(self, test):
        self.success_count += 1

        TestResult.addSuccess(self, test)
        output = self.complete_output()

        self.result.append((0, test, output, ''))
        if self.verbosity > 1:
            sys.stderr.write('OK ')
            sys.stderr.write(str(test))
            sys.stderr.write('\n')
        else:
            sys.stderr.write('.')

    def addFailure(self, test, err):
        self.failure_count += 1

        TestResult.addFailure(self, test, err)
        _, _exc_str = self.failures[-1]
        output = self.complete_output()

        self.result.append((1, test, output, _exc_str))
        if self.verbosity > 1:
            sys.stderr.write('Fail  ')
            sys.stderr.write(str(test))
            sys.stderr.write('\n')
        else:
            sys.stderr.write('F')

    def addError(self, test, err):
        self.error_count += 1

        TestResult.addError(self, test, err)
        _, _exc_str = self.errors[-1]
        output = self.complete_output()

        self.result.append((2, test, output, _exc_str))
        if self.verbosity > 1:
            sys.stderr.write('Error  ')
            sys.stderr.write(str(test))
            sys.stderr.write('\n')
        else:
            sys.stderr.write('E')

    def addSkip(self, test, err):
        self.skip_count += 1

        TestResult.addSkip(self, test, err)
        _, _exc_str = self.skipped[-1]
        output = self.complete_output()

        self.result.append((3, test, output, _exc_str))
        if self.verbosity > 1:
            sys.stderr.write('Skip  ')
            sys.stderr.write(str(test))
            sys.stderr.write('\n')
        else:
            sys.stderr.write('S')


class HTMLTestRunner(TemplateMixin):
    def __init__(self, stream=sys.stdout, verbosity=2, title=None, description=None,
                 theme=None, stylesheet=None, htmltemplate=None, javascript=None):
        self.stream = stream
        self.verbosity = verbosity

        self.start_time = datetime.datetime.now()
        self.duration = None

        self.title = title or self.DEFAULT_TITLE
        self.description = description or self.DEFAULT_DESCRIPTION

        theme = theme or 'default'
        self.htmltemplate = htmltemplate or f'{theme}.html'
        self.stylesheet = stylesheet or f'{theme}.css'
        self.javascript = javascript or f'{theme}.js'

    def run(self, test):
        result = _TestResult(self.verbosity)
        test(result)

        self.duration = datetime.datetime.now() - self.start_time
        data = self.generate_data(result)
        html = self.generate_report(data)

        print(f'\nTime Elapsed: {self.duration}', file=sys.stderr)
        result.pytestreport_data = data
        result.pytestreport_html = html

        return result

    def generate_report(self, data):
        html_template = self.get_html_template()
        output = html_template.render(**data)

        self.stream.write(output.encode('utf-8'))
        return output

    def generate_data(self, result):
        report_detail = self._generate_report_detail(result)
        report_summary = self.get_report_summary(result)
        report_summary['suite_count'] = report_detail['suite_count']
        return {
            'generator': 'PyTestReport %s' % __version__,
            'title': saxutils.escape(self.title),
            'description': saxutils.escape(self.description),
            'stylesheet': self.get_stylesheet(),
            'javascript': self.get_javascript(),
            'report_detail': report_detail,
            'report_summary': report_summary
        }

    def get_report_summary(self, result):
        start_time = str(self.start_time)[:19]
        duration = str(self.duration)
        status = {
            'pass': result.success_count,
            'fail': result.failure_count,
            'error': result.error_count,
            'skip': result.skip_count,
            'count': result.skip_count + result.error_count + result.failure_count + result.success_count
        }
        return {
            'start_time': start_time,
            'duration': duration,
            'status': status,
            'suite_count': 0
        }

    def get_stylesheet(self):
        with open(os.path.join(self.STYLESHEET_DIR, self.stylesheet), encoding='utf-8') as f:
            return f.read()

    def get_javascript(self):
        with open(os.path.join(self.JAVASCRIPT_DIR, self.javascript), encoding='utf-8') as f:
            return f.read()

    def get_html(self):
        with open(os.path.join(self.HTMLTEMPLATE_DIR, self.htmltemplate), encoding='utf-8') as f:
            return f.read()

    def get_html_template(self):
        html = self.get_html()
        return env.from_string(html)

    def _generate_report_detail(self, result):
        tests = []
        sorted_result = self.sort_result(result.result)
        for cid, (cls, cls_results) in enumerate(sorted_result, 1):
            # subtotal for a class
            np = nf = ne = ns = 0
            for n, t, o, e in cls_results:
                if n == 0:  # pass
                    np += 1
                elif n == 1:    # fail
                    nf += 1
                elif n == 2:    # error
                    ne += 1
                else:       # skip
                    ns += 1

            # format class description
            if cls.__module__ == "__main__":
                name = cls.__name__
            else:
                name = "%s.%s" % (cls.__module__, cls.__name__)
            doc = cls.__doc__.split("\n")[0] if cls.__doc__ else ""
            desc = '%s: %s' % (name, doc) if doc else name

            test = {
                'summary': {
                    'desc': desc,
                    'count': np + nf + ne + ns,
                    'pass': np,
                    'fail': nf,
                    'error': ne,
                    'skip': ns,
                    'cid': 'testclass%s' % cid,
                    'status': (ne and self.STATUS[2]) or (nf and self.STATUS[1]) or (ns and self.STATUS[3]) or self.STATUS[0]
                }, 'detail': []
            }

            for tid, (n, t, o, e) in enumerate(cls_results, 1):
                test['detail'].append(self._generate_report_test(cid, tid, n, t, o, e))

            tests.append(test)

        return {
            'tests': tests,
            'count': str(result.success_count + result.failure_count + result.error_count + result.skip_count),
            'pass': str(result.success_count),
            'fail': str(result.failure_count),
            'error': str(result.error_count),
            'skip': str(result.skip_count),
            'suite_count': len(sorted_result)
        }

    def _generate_report_test(self, cid, tid, n, t, o, e):
        # e.g. 'pt1.1', 'ft1.1', etc
        has_output = bool(o or e)
        tid = 'test%s.%s.%s' % (self.STATUS[n], cid, tid)
        name = t.id().split('.')[-1]
        doc = t.shortDescription() or ""
        desc = doc and ('%s: %s' % (name, doc)) or name

        uo = o if isinstance(o, str) else o.decode('utf-8')
        ue = e if isinstance(e, str) else e.decode('utf-8')

        return {
            'has_output': has_output,
            'tid': tid,
            'desc': desc,
            'output': saxutils.escape(str(uo) + str(ue)),
            'status': self.STATUS[n],
            'status_code': n
        }

    @staticmethod
    def sort_result(result_list):
        rmap = {}
        for n, t, o, e in result_list:
            cls = t.__class__
            rmap.setdefault(cls, []).append((n, t, o, e))
        return rmap.items()



class TestProgram(unittest.TestProgram):


    def runTests(self):
        fp = None
        if self.testRunner is None:
            fp = open('report/GLR Automation Test Report.html', 'wb')
            self.testRunner = HTMLTestRunner(fp, verbosity=self.verbosity)
        unittest.TestProgram.runTests(self)
        if fp:
            fp.close()


main = TestProgram

if __name__ == "__main__":
    main(module=None)



import os
from subprocess import call

from .HTMLTestRunner import HTMLTestRunner
from .SendEmail import SendMail

__all__ = ["make_report", "make_image", "send_report"]


def make_report(stream, data, theme=None,
                stylesheet=None, htmltemplate=None, javascript=None):
    test_runner = HTMLTestRunner(stream, theme=theme, stylesheet=stylesheet,
                                htmltemplate=htmltemplate, javascript=javascript)
    data['stylesheet'] = test_runner.get_stylesheet()
    data['javascript'] = test_runner.get_javascript()
    return test_runner.generate_report(data)


def make_image(html_file, image_file, img_width="1350px"):

    js = os.path.join(os.path.dirname(__file__), "static", "js", "capture.js")
    cmds = ["phantomjs", js, html_file, image_file, img_width]
    output = call(cmds)
    if output == 0:
        print('make capture success!')
    else:
        print('make capture failed!')


def send_report(subject, html_file, image_file, frm, to, cc=[]):
    """
    发送邮件报告
    :param subject: 邮件主题
    :param html_file: 邮件html附件
    :param image_file: 邮件正文图片
    :param frm: 发件人邮箱
    :param to: 收件人邮箱列表
    :param cc: 抄送人邮箱列表
    :return:
    """
    send_mail = SendMail(frm=frm, to=to, cc=cc)
    attaches = [(html_file, 'text', 'html', os.path.basename(html_file), 1)]
    send_mail.send_as_image(subject, image_file, attaches=attaches)
